import java.io.*;
import java.net.*;
import java.security.*;
import javax.crypto.*;

public class CoordinatorChildThreadSocketHandler extends Thread {

    private Socket                 socket;
    private CoordinatorSecuredComm csc;
    private String                 host;
    private int                    port;
    private CoordinatorServerInfo  csi;
    private int                    id;
    private int                    total;
    private int                    available;

    public CoordinatorChildThreadSocketHandler() {}
    
    public CoordinatorChildThreadSocketHandler(Socket s, CoordinatorServerInfo c,
                     CoordinatorKeyPair ckp) {
        socket = s;
        csi = c;
        csc = new CoordinatorSecuredComm(socket, ckp);
    }

    public void run() {
        try {

            WFMessage wfm;

            // initialize the socket connection and all message control objects
            int i = 0;
            if (!csc.init()) {
                System.err.println("CoordinatorSecuredComm.init(): failed.");
                System.exit(1);
            }

            if (csc.receive() && (csc.getMessageType() == 2)) {
                wfm= (WFMessage)csc.getMessage();
                if (wfm.type == 6) {
                    try {
                        // Host information might be passed through Socket, but I
                        // choose to obtain the host from socket 
                        host = (socket.getInetAddress()).getHostName();
                        port = Integer.parseInt(wfm.msg);
                    } catch (Exception e) {
                        System.err.println("CoordinatorChildThreadSocketHandler error: " +
                                           "failed to recognize server's port");
                    }
                } else {
                    System.err.println("CoordinatorChildThreadSocketHandler error: " +
                                       "failed to verify server's reponse");
                }
            } else {
                System.err.println("CoordinatorChildThreadSocketHandler error: " +
                                   "failed to authenticate message type");
            }

            boolean done = false;
            while (!done) {
                if (csc.receive() && (csc.getMessageType() == 2)) {
                    wfm = (WFMessage)csc.getMessage();
                    if (wfm.type == 7) {
                        try {
                            id = Integer.parseInt(wfm.msg);
                            done = true;
                        } catch (Exception e) {
                            System.err.println("CoordinatorChildThreadSocketHandler error: " +
                                               "failed to verify server's reponse");
                        }
                    }
                }
            }

            // put the server information into a hashtable class
            csi.putServer(id, host, port);

            done = false;
            while (!done) {
                if (csc.receive() && (csc.getMessageType() == 2)) {
                    wfm = (WFMessage)csc.getMessage();
                    if (wfm.type == 9) {
                        try {
                            total = Integer.parseInt(wfm.msg);
                            done = true;
                        } catch (Exception e) {
                            System.err.println("CoordinatorChildThreadSocketHandler error: " +
                                           "failed to verify server's reponse");
                        }
                    }
                } else {
                    System.err.println("CoordinatorChildThreadSocketHandler error: " +
                                       "failed to authenticate message type");
                }
            }

            csi.putServerStatus(id, total);

            while (true) {
                if (csc.receive()) {
                    wfm = (WFMessage)csc.getMessage();
                    if (wfm.type == 8) {
                        try {
                            available = Integer.parseInt(wfm.msg);
                            csi.updateServerStatus(id, total, available);
                        } catch (Exception e) {
                            System.err.println("CoordinatorChildThreadSocketHandler error: " +
                                               "failed to recognize server's availability");
                        }
                    }
                } else {
                    break;
                }
            }
        } catch (Exception ex) {
            System.err.println("CoordinatorChildThreadSocketHandler Exception-> " + ex);
            csi.removeServer(id);
        }
        csc.done();
    }
}
