import java.io.*;
import java.net.*;
import java.security.*;
import javax.crypto.*;

public class CoordinatorThreadSocketHandler extends Thread {

    private Socket                 socket;
    private CoordinatorServerInfo  csi;
    private ObjectOutputStream     oos;

    public CoordinatorThreadSocketHandler() {}
    
    public CoordinatorThreadSocketHandler(Socket s, CoordinatorServerInfo c) {
        socket = s;
        csi = c;
        try {
            oos = new ObjectOutputStream(socket.getOutputStream());
            oos.flush();
        } catch (Exception e) {
            System.err.println("CoordinatorThreadSocketHandler() Exception-> " + e);
        }
    }

    public void run() {
        try {

            // obtain available server from the local server pool
            CoordinatorServer cs = csi.getAvailableServer();

            // send an available server information to Client
            HostPortMessage hpm = new HostPortMessage(cs.host, cs.port);
            oos.writeObject(hpm);
            oos.flush();

            // check if there exist an available server
            if (cs.id < 0) {
                System.err.println("No server available.");
                return;
            }
        } catch (Exception ex) {
            System.err.println("CoordinatorThreadSocketHandler: run() Exception-> " + ex);
        }

        try {
            oos.close();
            socket.close();
        } catch (Exception e) {
            System.err.println("CoordinatorThreadSocketHandler: run() Exception-> " + e);
        }
    }
}
