import java.io.*;
import java.security.*;
import java.security.spec.*;
import java.security.interfaces.*;
import javax.crypto.*;

public class ClientRegHandShaking {

    private ObjectInputStream   ois;
    private ObjectOutputStream  oos;
    private PublicKey           clientregPublicKey;
    private PublicKey           upserverPublicKey;
    private PrivateKey          clientregPrivateKey;
    private SecretKey           secretKey;
    private ClientRegKeyPair       skp;

    public ClientRegHandShaking(ObjectInputStream in, ObjectOutputStream out,
                             ClientRegKeyPair sk) {
        ois = in;
        oos = out;
        skp = sk;
    }

    public boolean init() {
        try {
            // Initialize the DH protocol and exchange the secret key for cipher
            ClientRegDHKeyAgreement dhKey = new ClientRegDHKeyAgreement(ois, oos);
            secretKey = dhKey.agree();

            clientregPublicKey = skp.getPublicKey();
            clientregPrivateKey = skp.getPrivateKey();
            byte[] clientregEncodedPublicKey = skp.getEncodedPublicKey();

            // send ClientReg's public key to UPServer
            oos.writeObject(clientregEncodedPublicKey);
            oos.flush();
        
            // obtain UPServer's public key
            // This will be replaced by reading from upserver's certificate

            byte[] upserverEncodedPublicKey = (byte[])ois.readObject();

            // st = toHexString(upserverEncodedPublicKey);
            // System.err.println("UPServer's Encoded PublicKey: " + st);

            KeyFactory clientregKeyFac = KeyFactory.getInstance("DSA");
            X509EncodedKeySpec x509KeySpec = new X509EncodedKeySpec(upserverEncodedPublicKey);
            upserverPublicKey = clientregKeyFac.generatePublic(x509KeySpec);

            return true;
        } catch (Exception ex) {
            System.err.println("ClientRegHandShaking Exception-> " + ex);
            return false;
        }
    }

    public boolean init_() {
        try {

            // Initialize the DH protocol and exchange the secret key for cipher
            ClientRegDHKeyAgreement dhKey = new ClientRegDHKeyAgreement(ois, oos);
            secretKey = dhKey.agree();

            clientregPublicKey = skp.getPublicKey();
            clientregPrivateKey = skp.getPrivateKey();
            byte[] clientregEncodedPublicKey = skp.getEncodedPublicKey();

            // obtain UPServer's public key
            // This will be replaced by reading from upserver's certificate

            byte[] upserverEncodedPublicKey = (byte[])ois.readObject();

            // send ClientReg's public key to UPServer
            oos.writeObject(clientregEncodedPublicKey);
            oos.flush();

            // st = toHexString(upserverEncodedPublicKey);
            // System.err.println("UPServer's Encoded PublicKey: " + st);

            KeyFactory clientregKeyFac = KeyFactory.getInstance("DSA");
            X509EncodedKeySpec x509KeySpec = new X509EncodedKeySpec(upserverEncodedPublicKey);
            upserverPublicKey = clientregKeyFac.generatePublic(x509KeySpec);

            return true;
        } catch (Exception ex) {
            System.err.println("ClientRegHandShaking Exception-> " + ex);
            return false;
        }
    }

    public PublicKey getClientRegPublicKey() {
        return this.clientregPublicKey;
    }

    public PublicKey getUPServerPublicKey() {
        return this.upserverPublicKey;
    }

    public PrivateKey getClientRegPrivateKey() {
        return this.clientregPrivateKey;
    }

    public SecretKey getSecretKey() {
        return this.secretKey;
    }

    public void byte2hex(byte b, StringBuffer buf) {
        char hexChars[] = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};

        int low = (b & 0x0f);
        int high = (b & 0xf0) >> 4;

        buf.append(hexChars[high]);
        buf.append(hexChars[low]);
    }

    /*
     * Converts a byte array to hex string
     */
    public String toHexString(byte[] block) {
        StringBuffer buf = new StringBuffer();

        int len = block.length;

        for (int i = 0; i < len; i++) {
             byte2hex(block[i], buf);
             if (i < len-1) {
                 buf.append(":");
             }
        }
        return buf.toString();
    }
}
