import java.io.*;
import java.net.*;
import java.security.*;
import javax.crypto.*;

public class ClientRegSecuredComm {
    private Socket                   socket;
    private SecretKey                secretKey;
    private ClientRegHandShaking     crhs;
    private ObjectInputStream        ois;
    private ObjectOutputStream       oos;
    private ClientRegKeyPair         crkp;
    private String                   cipherAlgorithm = "DES/ECB/PKCS5Padding";


    public ClientRegSecuredComm(Socket s, ClientRegKeyPair ck) {
        socket = s;
        crkp = ck;
    }

    public boolean init() {
        try {
            ois = new ObjectInputStream(socket.getInputStream());
            oos = new ObjectOutputStream(socket.getOutputStream());
            oos.flush();

            crhs = new ClientRegHandShaking(ois, oos, crkp);
            if (!crhs.init()) return false;

            secretKey = crhs.getSecretKey();
            return true;
        } catch (Exception ex) {
            System.err.println("ClientRegSecuredComm Exception-> " + ex);
            return false;
        }
    }

    public SecretKey getSecretKey() {
        return this.secretKey;
    }

    public void ssend(SerializedMessage sm) {
        Cipher cipher;
        try {
            cipher = Cipher.getInstance(cipherAlgorithm);
            cipher.init(Cipher.ENCRYPT_MODE, secretKey);
            SealedObject so = new SealedObject(sm, cipher);
            oos.writeObject(so);
            oos.flush();
        } catch (Exception e) {
            System.err.println("ClientRegSecuredComm: ssend() Exception-> " + e);
        }
        cipher = null;
    }

    public SerializedMessage sreceive() {
        try {
            Cipher cipher = Cipher.getInstance(cipherAlgorithm);
            SealedObject so = (SealedObject) ois.readObject();
            cipher.init(Cipher.DECRYPT_MODE, secretKey);
            SerializedMessage sm = (SerializedMessage) so.getObject(cipher);
            cipher = null;
            return sm;
        } catch (Exception e) {
            System.err.println("ClientRegSecuredComm: sreceive() Exception-> " + e);
        }
        return null;
    }

    public void done() {
        try { 
            socket.close();
            ois.close();
            oos.close();
        } catch (Exception ex) {
            System.err.println("ClientRegSecuredComm Exception-> " + ex);
        }
    }
}
