import java.io.*;
import java.net.*;


public class Server {

    private ServerInfo        si;
    private ServerKeyPair     skp;
    private ThreadGroup       tg;
    private String            serverID;
    private String            cHost;
    private int               cPort;
    private String            aHost;
    private int               aPort;
    private int               amPort;
    private ServerSecrets     ses;

    protected Server() {};

    protected void init(String id, String s, String p, String ah, String ap, String amp) {
        System.out.println("Generating key pair....");
        skp = new ServerKeyPair();
        tg = new ThreadGroup("Service_" + id);
        cHost = s;
        cPort = Integer.parseInt(p);
        aHost = ah;
        aPort = Integer.parseInt(ap);
        amPort = Integer.parseInt(amp);

        // negotiate the secrets between Authenticator and the current server
        ses = new ServerSecrets(id, aHost, amPort, skp);
        ses.init();
    }

    protected void run(String id, int port) {
        int i = 1;

        try {
            ServerSocket ss = new ServerSocket(port, 10);

            // Server-Coordinator communication socket
            Socket s = new Socket(cHost, cPort);
            si = new ServerInfo(s, skp, id, ss);
            si.init();
            System.out.println("Server " + id + " is ready");

            int sid = Integer.parseInt(id);
            for (;;) {
                Socket incoming = ss.accept();
                if (si.increase()) {
                    System.out.println("Spawning Service Thread " + i);
                    incoming.setSoTimeout(600000);
                    new ServerThreadSocketHandler(i, sid, incoming, si, skp, 
                                    tg, aHost, aPort, ses).start();
                    i++;
                }
            }
        } catch (Exception ex) {
            System.err.println("Server Exception->" + ex);
        }
    }

    public static void main(String[] args) {

        if (args.length != 14) {
            System.err.println("java Server -ID # -Port # -CHost hostname -CPort # -AHost hostname -APort # -AMPort #\n" +
                               "    -ID:     Server ID\n" +
                               "    -Port:   Server port\n" +
                               "    -CHost:  Coordinator hostname\n" +
                               "    -CPort:  Coordinator port\n" +
                               "    -AHost:  Authenticator hostname\n" +
                               "    -APort:  Authenticator port\n" +
                               "    -AMPort: Authenticator secret port");
            System.exit(1);
        }

        int i = 0;
        while (!args[i++].equals("-ID")) {};
        String id = args[i];

        i = 0;
        while (!args[i++].equals("-Port")) {};
        int port = Integer.parseInt(args[i]);

        i = 0;
        while (!args[i++].equals("-CHost")) {};
        String chost = args[i];

        i = 0;
        while (!args[i++].equals("-CPort")) {};
        String cport = args[i];

        i = 0;
        while (!args[i++].equals("-AHost")) {};
        String ahost = args[i];

        i = 0;
        while (!args[i++].equals("-APort")) {};
        String aport = args[i];

        i = 0;
        while (!args[i++].equals("-AMPort")) {};
        String amport = args[i];

        System.out.println("Server " + id + " is being started at " + port);

        Server s = new Server();
        s.init(id, chost, cport, ahost, aport, amport);
        s.run(id, port);
    }
}
