import java.io.*;
import java.net.*;
import java.security.*;
import javax.crypto.*;


public class ServerThreadSocketHandler extends Thread {

    private int                    id;
    private Socket                 socket;
    private ServerSecuredComm      ssc;
    private ServerInfo             si;
    private String                 AHost;
    private int                    APort;
    private ServerSecrets          ss;
    private String                 cipherAlgorithm = "DES/ECB/PKCS5Padding";

    public ServerThreadSocketHandler(int i, int j, Socket s, ServerInfo inf, 
                ServerKeyPair skp, ThreadGroup g, String ahost, int aport,
                ServerSecrets se) {
        super(g, Integer.toString(i));
        id = j;
        socket = s;
        si = inf;
        ssc = new ServerSecuredComm(socket, skp);
        AHost = ahost;
        APort = aport;
        ss = se;
    }

    protected boolean verification(String u, String p) {
        Socket socket;
        ObjectOutputStream oos;
        ObjectInputStream ois;

        try {
            // open connection to Authenticator to verify username/password

            socket = new Socket(AHost, APort);
            ois = new ObjectInputStream(socket.getInputStream());
            oos = new ObjectOutputStream(socket.getOutputStream());
            oos.flush();
        } catch (Exception ex) {
            System.err.println("ServerThreadSocketHandler: verification() Exception-> " + ex);
            ois= null;
            oos= null;
            socket = null;
            return false;
        }

        try {
            Cipher cipher = Cipher.getInstance(cipherAlgorithm);
            cipher.init(Cipher.ENCRYPT_MODE, ss.getSecretKey());
            WFUserPassMessage wfupm = new WFUserPassMessage(u, p);
            SealedObject so = new SealedObject(wfupm, cipher);
            WFVerification wfv = new WFVerification(id, so);

            oos.writeObject(wfv);
            oos.flush();

            WFMessage wfm = (WFMessage) ois.readObject();

            ois.close();
            oos.close();
            socket.close();

            if (wfm.type == 1) return true;
            return false;
        } catch (Exception ex) {
            System.err.println("ServerThreadSocketHandler: verification() Exception-> " + ex);
        } 

        try {
            ois.close();
            oos.close();
            socket.close();
        } catch (Exception ex) {
            System.err.println("ServerThreadSocketHandler: verification() Exception-> " + ex);
        }
        return false;
    }

    public void run() {
        try {

            // initialize the socket connection and all message control objects
            int i = 0;
            if (!ssc.init()) {
                System.err.println("ServerSecuredComm.init(): failed");
                ssc.done();
            }

            // check the message type 
            if (ssc.receive()) {
                WFUserPassMessage wfupm = (WFUserPassMessage)ssc.getMessage();
                if (verification(wfupm.username, wfupm.password)) {
              
                    // get the dynamic URL from WebFlow server and retrun
                    // to the client
                    ssc.setupMessage(1, "http://dante.npac.syr.edu:1965/");
                } else {
                    System.err.println("ServerThread: authentication failed");
                    ssc.setupMessage(0, "verification failed");
                }
                ssc.send();
            }
        } catch (Exception ex) {
            System.err.println("ServerThread Exception-> " + ex);
        }
        ssc.done();
        si.decrease();
    }
}
