import java.io.*;
import java.net.*;
import java.security.*;
import javax.crypto.*;

public class UPServerThreadSocketHandler extends Thread {

    private Socket                   socket;
    private UPServerSecuredComm      upssc;
    private UPServerPool             upsp;

    public UPServerThreadSocketHandler(Socket s, UPServerPool p,
                     UPServerKeyPair kp) {
        socket = s;
        upsp = p;
        upssc = new UPServerSecuredComm(socket, kp);
    }

    public void run() {
        try {
            if (!upssc.init()) {
                System.err.println("UPServerSecuredComm.init(): failed.");
            }

            // receive client's information WFUserInfo
            WFUserInfo wfui = (WFUserInfo)upssc.sreceive();
          
            // send client's information back to client
            upssc.ssend(wfui);

            // receive the acknowledge from the client
            if (((WFMessage)upssc.sreceive()).type == 1) {
                if (upsp.put(wfui)) {
                    upssc.ssend(new WFMessage((int)1, ""));
                } else {
                    upssc.ssend(new WFMessage((int)0, ""));
                }
            } else {
                upssc.ssend(new WFMessage((int)0, ""));
            }
        } catch (Exception ex) {
            System.err.println("UPServerThreadSocketHandler Exception-> " + ex);
        }
        upssc.done();
    }
}
